package org.rcisoft.business.bchapter.controller;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.rcisoft.business.bchapter.dto.QueryScoreListDTO;
import org.rcisoft.business.bchapter.dto.ScoreInfoDTO;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.enums.IsCompleteEnum;
import org.rcisoft.business.bchapter.service.BChapterService;
import org.rcisoft.business.bsl.service.BSlService;
import org.rcisoft.common.component.Global;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.enums.RoleEnum;
import org.rcisoft.core.enums.RoleFlagEnum;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.role.dao.SysRoleRepository;
import org.rcisoft.sys.role.entity.SysRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.async.WebAsyncTask;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.Callable;

/**
 * Created by gwf on 2017-7-21 15:08:47.
 */
@RestController
@RequestMapping("/BChapter")
@Slf4j
public class BChapterController extends PaginationController<BChapter> {
    @Autowired
    private BChapterService bChapterService;
    @Autowired
    private BSlService bSlService;

    @Autowired
    private SysRoleRepository sysRoleRepository;

    @Autowired
    private Global global;

    @ApiOperation(value="添加/编辑章节", notes="businessId为空时是添加方法，不为空时是修改方法")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterName", value = "章节名称", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pid", value = "上一级", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "classHour", value = "课时", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "experimentType", value = "实验类型,1:java单文件 2:java工程 3:html单文件 4:html工程", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonOrSl", value = "课程或开课;0：开课  1：课程", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "isTest", value = "1:实验  2:视频   3:PPT  4:pdf   5:资料   6试题", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonId", value = "课程ID(课程ID和开课ID只能存在一个)", required = false, dataType = "varchar")})
    @PreAuthorize("hasAnyRole('ROLE_1001','ROLE_1002')")
    @PostMapping(value = "/add")
    public Result add(@Valid BChapter bChapter,
                      BindingResult bindingResult
    ) {
        if (bindingResult.hasErrors()) {
            if (bChapter.getBusinessId() == null)
                throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR.getCode(),
                        bindingResult.getFieldError().getDefaultMessage());
        }
        PersistModel data = bChapterService.persist(bChapter,getToken());
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapter);
    }


    @ApiOperation(value="删除章节", notes="根据ID删除一条记录")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonOrSl", value = "区分课程开课 0开课，1课程", required = true, dataType = "varchar")})
    @PreAuthorize("hasAnyRole('ROLE_1001','ROLE_1002')")
    @PostMapping(value = "/remove")
    public Result remove(String id, String lessonOrSl) {
        PersistModel data = bChapterService.removeBChapter(id,lessonOrSl);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="唯一查找", notes="根据主键唯一查找任务书")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping(value = "/getMdFileById")
    public Result queryOne(String businessId) {
        String mdFile = bChapterService.selectOne(businessId).getMdFile();
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                mdFile);
    }

    @ApiOperation(value="上传图片", notes="上传任务书所需图片")
    @ApiImplicitParam(name = "image", value = "图片文件", required = true, dataType = "MultipartFile")
    @PreAuthorize("hasRole('ROLE_1002')")
    @PostMapping(value = "/uploadMdFileWithoutAuth")
    public Map<String, Object> uploadMdFile(@RequestParam(name = "editormd-image-file") MultipartFile image) {
        String path = global.getMD_FILE_LOCATION();
        String basePath = global.getBASE_UPLOAD_SERVER_LOCATION();
        boolean type = false;
        if("1".equals(global.getIS_SERVER_LINUX())){
            type = true;
        }
        Map<String, Object> pic = UploadUtil.picImport(basePath, path, image, type);
        Map<String, Object> result = new HashMap<>();
        if (!(boolean) pic.get(UploadUtil.IS_SUCCESSS)) {
            result.put("success", 0);
            result.put("message", ResultServiceEnums.UPLOAD_FILE_ERROR.getMessage());
            result.put("url", "");
        } else {
            result.put("url", global.getSERVER_URL() + "/" + pic.get(UploadUtil.URL));
            result.put("success", 1);
            result.put("message", ResultServiceEnums.SUCCESS.getMessage());
        }
        return result;
    }


    @ApiOperation(value="分页查询章节", notes="根据条件分页查询章节")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonOrSl", value = "课程或开课;0：开课  1：课程", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonId", value = "课程ID(课程ID和开课ID只能存在一个)", required = false, dataType = "varchar")})
    @PostMapping(value = "/queryBChaptersByPagination")
    public GridModel queryBChaptersByPagination(BChapter param) {
        param.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bChapterService.queryBChaptersByPagination(getPaginationUtility(), param);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }


    @ApiOperation(value="根据条件查询章节", notes="根据条件查询章节")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课id", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonOrSl", value = "课程或开课;0：开课  1：课程", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonId", value = "课程ID(课程ID和开课ID只能存在一个)", required = false, dataType = "varchar")})
    @GetMapping(value = "/queryBChaptersWithoutAuth")
    public Result queryBChapters(BChapter param) {
        Map<String, Object> map = new HashedMap();
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        List<SysRole> role = sysRoleRepository.queryCodeByBusinessId(userId);
        if(role.size()>=1){
            if (role.get(0).getCode().equals(RoleEnum.TEACHER.getCode()))
                map.put("RoleFlag", RoleFlagEnum.TEACHER.getFlag());
            else {
                map.put("RoleFlag", RoleFlagEnum.STUDENT.getFlag());
                map.put("scoreList", bChapterService.queryScoreListBySlIdAndStuId(param.getSlId(), userId));
            }
        }
        map.put("chapterList", bChapterService.queryBChapters(param));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                map);
    }

    @ApiOperation(value="分数列表", notes="根据章节id获取分数列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "stuInfo", value = "学生学号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节主键", required = true, dataType = "varchar")})
    @GetMapping(value = "/queryScoreListByChapterId")
    public GridModel queryScoreListByChapterId(QueryScoreListDTO dto) {
        bChapterService.queryScoreListByChapterIdByPagination(getPaginationUtility(), dto);
        GridModel gridModel = getGridModelResponse();
        return gridModel;
    }

    @ApiOperation(value="批量打分", notes="批量打分")
    @ApiImplicitParams({@ApiImplicitParam(name = "chapterId", value = "章节id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "score", value = "分数", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentId", value = "学生id", required = true, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1002')")
    @PostMapping(value = "/markBatch")
    public Result markBatch(String scoreInfoList) {
        Gson gson = new Gson();
        List<ScoreInfoDTO> list = new ArrayList<ScoreInfoDTO>();
        try {
            list = gson.fromJson(scoreInfoList,
                    new TypeToken<List<ScoreInfoDTO>>() {
                    }.getType());
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR);
        }
        bChapterService.markScore(list,getToken());
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                scoreInfoList);
    }

    @ApiOperation(value="重新打分", notes="重新打分")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PreAuthorize("hasRole('ROLE_1002')")
    @PostMapping(value = "/reMark")
    public Result reMark(String id) {
        bChapterService.deleteScoerInfo(id);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }

    @ApiOperation(value="完成学习", notes="完成学习")
    @ApiImplicitParams({@ApiImplicitParam(name = "score", value = "分数", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "chapterId", value = "章节id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "filePath", value = "文件路径", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentId", value = "学生学号", required = true, dataType = "varchar")})
    @PreAuthorize("hasRole('ROLE_1003')")
    @PostMapping(value = "/complete")
    public Result complete(ScoreInfoDTO scoreInfoDTO) {
        scoreInfoDTO.setIsComplete(IsCompleteEnum.COMPLETE.getCode());
        scoreInfoDTO.setStudentId(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        bChapterService.updateIsComplete(scoreInfoDTO,getToken());
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                scoreInfoDTO);
    }


    private Callable<Result> timeOutCallBack(){

        Callable<Result> callback = (()-> {
                log.info("请求超时");
                Result result = new Result();
                result.setMessage("请求超时");
                return result;
        });
        return callback;
    }

    @ApiOperation(value="关闭che", notes="根据端口关闭che")
    @ApiImplicitParam(name = "port", value = "che启动的端口", required = true, dataType = "varchar")
    @PostMapping(value = "/cheStopWithoutAuth" )
    public WebAsyncTask<Result> cheStop(String port) throws Exception {
        Result result = new Result();
        log.info("主线程开始");
        Callable<Result> data = (() -> {
            log.info("副线程开始");
            /*action.cheStop(port);
            result.setCode(ResultCode.SUCCESS.code);
            result.setMessage(ResultServiceEnums.SUCCESS.getMessage());
            result.setData(null);*/
            log.info("副线程结束");
            return result;
        });
        WebAsyncTask<Result> webAsyncTask = new WebAsyncTask<>(300000, data);
        webAsyncTask.onTimeout(timeOutCallBack());
        log.info("主线程结束");
        return webAsyncTask;
    }

    @ApiOperation(value="提交代码", notes="提交代码")
    @ApiImplicitParams({@ApiImplicitParam(name = "port", value = "端口", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "stuCode", value = "学生学号", required = false, dataType = "varchar")})
    @PostMapping(value = "/pushCode")
    public Result pushCode(String port,
                                 String slCode,
                                 String stuCode) {
        /*try {
            action.pushCode(
                    global.getGIT_LAB_PROJECT_PREFIX() + slCode,
                    port,
                    global.getGIT_LAB_STU_PREFIX() + stuCode
            );*/
            return Result.builder(new PersistModel(1),
                    ResultServiceEnums.SUCCESS.getMessage(),
                    MessageConstant.MESSAGE_ALERT_ERROR,
                    null);
        /*} catch (Exception e) {
            throw new ServiceException(ResultServiceEnums.PUSH_ERROR);
        }*/

    }

    @ApiOperation(value="检出分支", notes="检出分支")
    @ApiImplicitParams({@ApiImplicitParam(name = "port", value = "端口", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slCode", value = "课程编号", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "stuCode", value = "学生学号", required = false, dataType = "varchar")})
    @PostMapping(value = "/checkoutCode")
    public Result checkoutCode(String port,
                                     String slCode,
                                     String stuCode) throws IOException, InterruptedException {
        /*action.checkoutBranch(
                global.getGIT_LAB_PROJECT_PREFIX() + slCode,
                port,
                global.getGIT_LAB_STU_PREFIX() + stuCode);*/
        return Result.builder(new PersistModel(1),
                ResultServiceEnums.SUCCESS.getMessage(),
                MessageConstant.MESSAGE_ALERT_ERROR,
                null);
    }


    @PostMapping(value = "/testCountDown")
    public String testCountDown() throws Exception {
        return getPort();
    }

    @ApiOperation(value="刷新che启动时间", notes="根据端口刷新che启动时间")
    @ApiImplicitParam(name = "port", value = "che启动的端口", required = true, dataType = "varchar")
    @PostMapping(value = "/refreshChe")
    public Result refreshChe(String port) {
        /*CountDown countDown = CountDown.portMap.get(port);
        if (countDown.getMidTime() > 300)
            throw new ServiceException(ResultServiceEnums.TEST_NOT_OVER);
        countDown.setMidTime(Integer.parseInt(global.getCHE_START_TIME()));
        return Result.builder(new PersistModel(1),ResultServiceEnums.SUCCESS.getMessage(),countDown.getEndTime().getTime());*/
        return Result.builder(new PersistModel(1),
                ResultServiceEnums.SUCCESS.getMessage(),
                MessageConstant.MESSAGE_ALERT_ERROR,
                new Date().getTime());
    }


    private String getPort() throws Exception {
        Random random = new Random();
        Integer port = 8000 + random.nextInt(1000);
        /*while (CountDown.portMap.containsKey(port))
            port = 8000 + random.nextInt(1000);*/
        return port.toString();
    }


    @ApiOperation(value="更改章节状态", notes="根据章节ID更改章节状态")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar")})
    @PostMapping(value = "/updateChapterState")
    public Result updateChapterState(BChapter bChapter){
        int data = bChapterService.updateChapterState(bChapter);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapter);
    }


    @ApiOperation(value="更改全部章节状态", notes="根据开课ID更改章节状态")
    @ApiImplicitParam(name = "slId", value = "开课ID", required = true, dataType = "varchar")
    @PostMapping(value = "/updateAllChapterState")
    public Result updateAllChapterState(String slId){
        int data = bChapterService.updateAllChapterState(slId);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                slId);
    }

    @ApiOperation(value="章节向上调顺序", notes="章节向上调顺序")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonId", value = "课程ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "sort", value = "当前顺序", required = true, dataType = "Integer")})
    @PostMapping(value = "/updateTopChapter")
    public Result updateTopChapter(BChapter bChapter){
        int data = bChapterService.updateTopChapter(bChapter);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapter);
    }

    @ApiOperation(value="章节向下调顺序", notes="章节向下调顺序")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "pid", value = "父节点ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonId", value = "课程ID", required = false, dataType = "varchar"),
            @ApiImplicitParam(name = "sort", value = "当前顺序", required = true, dataType = "Integer")})
    @PostMapping(value = "/updateDownChapter")
    public Result updateDownChapter(BChapter bChapter){
        int data = bChapterService.updateDownChapter(bChapter);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapter);
    }


    @ApiOperation(value="更改是否智评状态", notes="根据章节ID更改章节是否智评的状态")
    @ApiImplicitParam(name = "businessId", value = "章节ID", required = true, dataType = "varchar")
    @PostMapping(value = "/updateAutomatic")
    public Result updateAutomatic(String businessId){
        int data = bChapterService.updateAutomatic(businessId);
        return Result.builder(new PersistModel(data),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @ApiOperation(value="根据章节ID查询任务书", notes="根据章节ID查询任务书")
    @ApiImplicitParam(name = "businessId", value = "章节ID", required = true, dataType = "varchar")
    @GetMapping(value = "/queryMdFileByChapterId")
    public Result queryMdFileByChapterId(String businessId){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                bChapterService.queryMdFileByChapterId(businessId));
    }


    @ApiOperation(value="查询学生章节列表", notes="根据学生id查询学生章节列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "slId", value = "开课id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "lessonOrSl", value = "课程或开课;0：开课  1：课程", required = true, dataType = "varchar")})
    @GetMapping(value = "/queryBChaptersByStudent")
    public Result queryBChaptersByStudent(BChapter param) {
        Map<String, Object> map = new HashedMap();
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        param.setUserId(userId);
        map.put("RoleFlag", RoleFlagEnum.STUDENT.getFlag());
        map.put("scoreList", bChapterService.queryScoreListBySlIdAndStuId(param.getSlId(), userId));
        map.put("chapterList", bChapterService.queryBChaptersByStudent(param));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                map);
    }


    @ApiOperation(value="打分", notes="根据章节ID和学号进行打分")
    @ApiImplicitParams({@ApiImplicitParam(name = "chapterId", value = "章节ID", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "slId", value = "开课id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "studentCode", value = "学号", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "score", value = "分数", required = true, dataType = "varchar"),})
    @PreAuthorize("hasRole('ROLE_1002')")
    @PostMapping(value = "/Mark")
    public Result Mark(String chapterId,String slId,String studentCode,String score) {
        return Result.builder(new PersistModel(bChapterService.Mark(chapterId,slId,studentCode,score)),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                studentCode);
    }
}
