package org.rcisoft.business.bchapter.dao;

import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bchapter.dto.QueryScoreListDTO;
import org.rcisoft.business.bchapter.dto.ScoreInfoDTO;
import org.rcisoft.business.bchapter.dto.StuScoreDTO;
import org.rcisoft.business.bchapter.entity.BChapter;
import org.rcisoft.business.bchapter.vo.StudentChapterScoreVO;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.security.access.method.P;
import org.springframework.stereotype.Repository;
import tk.mybatis.mapper.common.Mapper;

import java.util.List;
import java.util.Map;

/**
* Created with gwf on 2017-7-21 15:08:47.
*/
@Repository
public interface BChapterRepository extends BaseMapper<BChapter> {

    /**
    * 根据条件查找
    * @param param
    * @return
    */
    @Select("<script>select * from b_chapter  " +
            "where del_flag = 0 and flag = 1 and pid = -1 " +
            "<if test=\"slId!=null and slId != ''\">\n" +
            "   and sl_id = #{slId}\n" +
            "</if>" +
            "<if test=\"lessonId!=null and lessonId != ''\">\n" +
            "   and lesson_id = #{lessonId}\n" +
            "</if>" +
            "and lesson_or_sl = #{lessonOrSl} " +
            "order by sort asc</script>")
    @ResultMap(value = "SupperChildListResultMap" )
    List<BChapter> queryBChapters(Map<String, Object> param);

    /**
     * 根据父节点查找子节点列表
     * @param pid
     * @return
     */
    @Select("<script>SELECT\n" +
            " bc.*,\n" +
            " bf.video_url AS videoUrl,\n" +
            " bf.file_url AS fileUrl,\n" +
            " bf.ppt_url AS pptUrl,\n" +
            " bf.pdf_url AS pdfUrl,\n" +
            " (SELECT count(*) FROM b_r_student_chapter brs WHERE brs.chapter_id = bc.business_id AND brs.score = - 1) AS studentNum \n" +
            " FROM\n" +
            " b_chapter bc\n" +
            " LEFT JOIN b_file bf ON bf.chapter_id = bc.business_id " +
            " \n" +
            " WHERE\n" +
            " bc.del_flag = 0\n" +
            " AND bc.pid = #{pid}\n" +
            " ORDER BY\n" +
            " bc.sort asc</script>")
    @ResultMap(value = "SupperChildListResultMap" )
    List<BChapter> queryBChaptersByPid(String pid);

    /**
     * 根据章节id查找该章节学生分数列表
     * @param
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt1.chapter_name,t5.class_name,t3.`code`,t6.`name`,t2.score,t3.business_id as sId,t2.business_id as scoreId\n" +
            "FROM\n" +
            "\tb_chapter t1\n" +
            "\tLEFT JOIN b_r_student_chapter t2 ON t2.chapter_id = t1.`business_id`\n" +
            "\tLEFT JOIN b_student t3 ON t2.student_id = t3.`business_id`\n" +
            "\tLEFT JOIN b_r_class_student t4 ON t3.`code` = t4.student_code\n" +
            "\tLEFT JOIN b_class t5 ON t5.`code` = t4.class_code\n" +
            "\tLEFT JOIN s_user t6 ON t6.login_name = t3.`code`\n" +
            "WHERE t1.del_flag='0' and t1.flag='1' and t2.is_complete='1' " +
            "\nand t2.`score`  <if test=\"hasMark\">!=</if> <if test=\"!hasMark\">=</if> -1" +
            "\tand t1.business_id = #{dto.chapterId}\n" +
            "\tand (" +
            "(t3.code like #{dto.stuInfo} or t6.name like #{dto.stuInfo})\n" +
            "\tor " +
            "(t5.code like #{dto.stuInfo} or t5.class_name like #{dto.stuInfo})" +
            ")</script>")
    @ResultMap("QueryScoreListByChapterIdResultMap")
    List<StudentChapterScoreVO> queryScoreListByChapterId(@Param("dto") QueryScoreListDTO dto, @Param("hasMark") boolean hasMark);

    /**
     * 根据课程id和学生id查找该学生所学课程分数列表
     * @param slid
     * @param stuid
     * @return
     */
    @Select("<script>SELECT\n" +
            "\tt2.chapter_id as chapterId,\n" +
            "\tt2.score\n" +
            "FROM\n" +
            "\tb_chapter t1\n" +
            "LEFT JOIN b_r_student_chapter t2 ON t1.business_id = t2.chapter_id\n" +
            "LEFT JOIN b_sl t3 ON t1.sl_id = t3.business_id\n" +
            "LEFT JOIN b_student t4 ON t2.student_id = t4.business_id\n" +
            "LEFT JOIN s_user t5 ON t5.login_name = t4.code\n" +
            "WHERE\n" +
            "\tt3.business_id = #{slid}\n" +
            "AND t5.business_id = #{stuid} " +
            "AND t1.del_flag = 0 and t1.flag = 1</script>")
    List<StuScoreDTO> selectScoreListBySlIdAndStuId(@Param("slid") String slid, @Param("stuid") String stuid);

    @Delete("<script>delete from b_r_student_chapter where chapter_id = #{chapterId}</script>")
    int deleteStudentChapter(String chapterId);

    /**
     * 根据章节id和学生id获取主键
     * @param chapterId
     * @param studentId
     * @return
     */
    @Select("<script>select business_id from b_r_student_chapter " +
            "where chapter_id = #{chapterId} and student_id = #{studentId}</script>")
    String selectIdByStudentIdAndChapterId(@Param("chapterId") String chapterId, @Param("studentId") String studentId);

    /**
     * 插入分数
     * @param scoreInfoDTO
     * @return
     */
    @Insert("</script>insert into b_r_student_chapter(business_id,chapter_id,student_id,score,sl_id) values(#{businessId},#{chapterId},#{studentId},#{score},#{slId})</script>")
    int insertScoreInfo(ScoreInfoDTO scoreInfoDTO);

    /**
     * 更新分数
     * @param scoreInfoDTO
     * @return
     */
    @Update("<script>update b_r_student_chapter set chapter_id=#{chapterId},student_id=#{studentId},score=#{score} where business_id = #{businessId}</script>")
    int updateScoreInfo(ScoreInfoDTO scoreInfoDTO);

    /**
     * 更新完成状态
     * @param scoreInfoDTO
     * @return
     */
    @Insert("<script>insert into b_r_student_chapter(business_id,student_id,chapter_id,is_complete,sl_id,score) values (#{businessId},#{studentId},#{chapterId},#{isComplete},#{slId},#{score})</script>")
    int updateIsComplete(ScoreInfoDTO scoreInfoDTO);

    /**
     * 删除分数记录
     * @param id
     * @return
     */
    @Update("<script>update b_r_student_chapter set score = -1 where  business_id = #{businessId}</script>")
    int deleteScoerInfo(String id);

    @Select("<script>select * from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and lesson_id = #{lessonId}</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BChapter> queryChapterByLessonId(String lessonId);

    @Insert("<script>INSERT INTO b_chapter " +
            "(create_date,create_by,update_date,update_by," +
            "del_flag,flag,business_id," +
            "sl_id,chapter_name,chapter_level,pid," +
            "chapter_url,md_file,is_test,lesson_or_sl,class_hour,sort)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.createDate},#{item.createBy},#{item.updateDate},#{item.updateBy}," +
            "#{item.delFlag},#{item.flag},#{item.businessId}," +
            "#{item.slId},#{item.chapterName},#{item.chapterLevel},#{item.pid}," +
            "#{item.chapterUrl},#{item.mdFile},#{item.isTest},#{item.lessonOrSl},#{item.classHour},#{item.sort})" +
            "</foreach></script>")
    int insertChapterList(List<BChapter> slList);

    @Select("<script>select chapter_name from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{pid}</script>")
    String selectChapterNameByPid(String pid);

    @Select("<script>select state from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{businessId}</script>")
    String selectStateById(String businessId);

    @Update("<script>update b_chapter set state = #{state} where  business_id = #{businessId}</script>")
    int updateStateById(BChapter bChapter);

    @Update("<script>update b_chapter set state = #{state} where  pid = #{businessId}</script>")
    int updateStateByPid(BChapter bChapter);

    @Select("<script>select DISTINCT state from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and sl_id = #{slId} order by state desc</script>")
    List<String> selectStateBySlId(String slId);

    @Update("<script>update b_chapter set state = #{state} where  sl_id = #{slId}</script>")
    int updateAllStateBySlId(@Param("slId") String slId, @Param("state") String state);

    @Select("<script>select DISTINCT IFNULL((select max(sort) from b_chapter where pid = #{pid} " +
            "and del_flag = 0 and flag = 1 " +
            "<if test=\"SlId !=null and SlId !=''\">AND sl_id = #{SlId} </if>" +
            "<if test=\"lessonId !=null and lessonId !=''\">AND lesson_id = #{lessonId} </if>),0)+1 as sort</script>")
    int selectMaxSort(@Param("pid") String pid,
                      @Param("SlId") String SlId,
                      @Param("lessonId") String lessonId);

    @Select("<script>select * from b_chapter where sort &lt; #{sort} " +
            "AND pid = #{pid} and del_flag = 0 and flag = 1 " +
            "<if test=\"slId !=null and slId !=''\">AND sl_id = #{slId} </if> " +
            "<if test=\"lessonId !=null and lessonId !=''\">AND lesson_id = #{lessonId} </if> " +
            "ORDER BY sort desc LIMIT 1</script>")
    @ResultMap(value = "BaseResultMap" )
    BChapter getlessChapter(BChapter bChapter);


    @Update("<script>update b_chapter set sort = #{sort} where business_id = #{businessId}</script>")
    int updateSortById(@Param("businessId") String businessId, @Param("sort") int sort);

    @Select("<script>select * from b_chapter where sort &gt; #{sort} " +
            "AND pid = #{pid} and del_flag = 0 and flag = 1 " +
            "<if test=\"slId !=null and slId !=''\">AND sl_id = #{slId} </if> " +
            "<if test=\"lessonId !=null and lessonId !=''\">AND lesson_id = #{lessonId} </if> " +
            "ORDER BY sort asc LIMIT 1</script>")
    @ResultMap(value = "BaseResultMap" )
    BChapter getBigChapter(BChapter bChapter);

    @Select("<script>select * from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and sl_id = #{slId}</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BChapter> queryChapterBySlId(String slId);

    @Select("<script>select automatic from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{businessId}</script>")
    String selectAutomatic(String businessId);

    @Update("<script>update b_chapter set automatic = #{automatic} where business_id = #{businessId}</script>")
    int updateAutomatic(@Param("automatic") String automatic, @Param("businessId") String businessId);

    @Select("<script>select md_file from b_chapter  " +
            "where del_flag = 0 and flag = 1 " +
            "and business_id = #{businessId}</script>")
    String queryMdFileByChapterId(String businessId);

    @Select("select CONCAT((SELECT sort from b_chapter where business_id = #{pid}),\n" +
            "'.'," +
            "(SELECT sort from b_chapter where business_id = #{businessId})) AS sort \n" +
            "from b_chapter where business_id = #{businessId} " +
            " and del_flag = 0 and flag = 1 ")
    String querySortByPidAndId(@Param("pid") String pid, @Param("businessId") String businessId);

    @Select("<script>select *,#{userId} AS stuId from b_chapter " +
            "where del_flag = 0 and flag = 1 and pid = -1 " +
            "<if test=\"slId!=null and slId != ''\">\n" +
            "   and sl_id = #{slId}\n" +
            "</if>" +
            "and lesson_or_sl = #{lessonOrSl} " +
            "order by sort asc</script>")
    @ResultMap(value = "SupperStudentChildListResultMap" )
    List<BChapter> queryBChaptersByStudent(BChapter param);


    @Select("<script>select bc.*,su.business_id as userId,su.`name` as studentName,su.login_name as stuId from b_chapter bc \n" +
            "LEFT JOIN b_r_student_chapter bsc on bc.business_id = bsc.chapter_id\n" +
            "LEFT JOIN b_student bs on bsc.student_id = bs.business_id\n" +
            "LEFT JOIN s_user su on su.login_name = bs.`code` \n" +
            "where bc.business_id = #{chapterId}</script>")
    @ResultMap(value = "BaseResultMap" )
    List<BChapter> queryBchapterStudentById(String chapterId);


    @Select("<script>SELECT brc.* from b_r_student_chapter brc\n" +
            "LEFT JOIN b_student bs ON brc.student_id = bs.business_id\n" +
            "where bs.`code` = #{studentCode}\n" +
            "AND brc.chapter_id = #{chapterId}</script>")
    ScoreInfoDTO queryScoreByChapterId(@Param("chapterId") String chapterId, @Param("studentCode") String studentCode);

    @Update("<script>update b_r_student_chapter " +
            "set score = #{score} " +
            "where student_id = #{studentId} " +
            "and chapter_id = #{chapterId}</script>")
    int remark(@Param("chapterId") String chapterId, @Param("studentId") String studentId, @Param("score") String score);

    @Insert("<script>insert into b_r_student_chapter(" +
            "business_id,student_id,chapter_id,is_complete,sl_id,score) " +
            "values (" +
            "#{businessId},#{studentId},#{chapterId},'1',#{slId},#{score})</script>")
    int insertStudentScore(String businessId, String chapterId, String slId, String studentId, String score);
}

