package org.rcisoft.business.bfile.service.impl;

import org.apache.commons.io.FileUtils;
import org.rcisoft.business.bfile.dao.BFileRepository;
import org.rcisoft.business.bfile.entity.BFile;
import org.rcisoft.business.bfile.service.BFileService;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.DelStatus;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.IdGen;
import org.rcisoft.core.util.PptToPdf;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by gaowenfneg on 2017-10-9 14:34:56.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class BFileServiceImpl implements BFileService {
    @Autowired
    private BFileRepository bFileRepository;
    @Autowired
    private Global global;

    @Override
    public BFile selectOne(String businessId){
        return bFileRepository.selectByPrimaryKey(businessId);
    }

    @Override
    public List<BFile> queryBVideosByPagination(PageUtil pageUtil, BFile model) {
        Map param = new HashMap<String,Object>();
        return bFileRepository.queryBVideos(param);
    }

    @Override
    public List<BFile> queryBVideos(BFile model) {
        Map param = new HashMap<String,Object>();
        return bFileRepository.queryBVideos(param);
    }


    @Override
    public PersistModel persist(BFile model, String token){
        UserUtil.setCurrentPersistOperation(model);
        int line = bFileRepository.insertSelective(model);
        return new PersistModel(line);
    }

    @Override
    public PersistModel removeBVideo(String id, String token) {
        BFile bFile = bFileRepository.selectByPrimaryKey(id);
        bFile.setDelFlag(DelStatus.DELETED.getStatus());
        UserUtil.setCurrentMergeOperation(bFile);
        int line = bFileRepository.updateByPrimaryKeySelective(bFile);
        return new PersistModel(line);
    }

    @Override
    public String uploadVideoToServer(List<MultipartFile> list, BFile bFile, String token) {

        String videoUrl = "";

        if (list.size()>0) {
            for (int i = 0; i < list.size(); ++i) {
                MultipartFile file = list.get(i);
                try {
                    //查询此节是否已有视频或文件
                    BFile video = bFileRepository.selectInfoByChapterId(bFile.getChapterId());

                    if (video!=null){//此节已有视频或文件，将已有的视频或文件放置到temp目录下，然后上传新视频或文件
                        if(bFile.getType().equals("0")&&video.getVideoUrl()!=null&&!video.getVideoUrl().equals("")){
                            this.removeVideoToTemp(video.getVideoUrl(),video.getVideoName(),bFile);//复制之前视频到temp文件下
                        }
                        if(bFile.getType().equals("1")&&video.getFileUrl()!=null&&!video.getFileUrl().equals("")){
                            this.removeVideoToTemp(video.getFileUrl(),video.getVideoName(),bFile);//复制之前文件到temp文件下
                        }
                        if(bFile.getType().equals("2")&&video.getPptUrl()!=null&&!video.getPptUrl().equals("")){
                            this.removeVideoToTemp(video.getPptUrl(),video.getVideoName(),bFile);//复制之前ppt到temp文件下
                        }
                        if(bFile.getType().equals("3")&&video.getPdfUrl()!=null&&!video.getPdfUrl().equals("")){
                            this.removeVideoToTemp(video.getPdfUrl(),video.getVideoName(),bFile);//复制之前pdf到temp文件下
                        }
                        BFile newVideo = this.uploadVideo(file, bFile);//上传新文件
                        newVideo.setBusinessId(video.getBusinessId());
                        UserUtil.setCurrentMergeOperation(newVideo);
                        bFileRepository.updateByPrimaryKeySelective(newVideo);//更新表中信息
                        if (bFile.getType().equals("0")){
                            videoUrl = newVideo.getVideoUrl();
                        }else if(bFile.getType().equals("1")){
                            videoUrl = newVideo.getFileUrl();
                        }else if(bFile.getType().equals("2")){
                            videoUrl = newVideo.getPptUrl();
                        }else{
                            videoUrl = newVideo.getPdfUrl();
                        }

                    }else{
                        BFile newVideo = this.uploadVideo(file, bFile);//上传文件
                        this.persist(newVideo,token);//插入表信息
                        videoUrl = newVideo.getVideoUrl();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    throw new RuntimeException(e);
                }
            }
        }
        return videoUrl;
    }




    /**
     * 将原视频复制到temp/video文件下，并删除原视频
     * @param url
     * @param bFile
     */
    private void removeVideoToTemp(String url, String fileName, BFile bFile) throws Exception{
        String path;
        String pathName;
        if(bFile.getLessonId()!=null&&!bFile.getLessonId().equals("")){
            path = global.getLESSON_LOCATION() + File.separator + bFile.getLessonId() + File.separator;
        }else{
            path = global.getSL_LOCATION() + File.separator + bFile.getSlId() + File.separator;
        }
        boolean ppt = false;
        if (bFile.getType().equals("0")){
            pathName = global.getVIDEO_LOCATION();//视频路径
        }else if(bFile.getType().equals("1")){
            pathName = global.getFILE_LOCATION();//文件路径
        }else if(bFile.getType().equals("2")){
            ppt = true;
            pathName = global.getPPT_LOCATION();//ppt路径
        }else{
            pathName = global.getPDF_LOCATION();//pdf路径
        }

        String tempPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + global.getTEMP_LOCATION() + File.separator
                + path + bFile.getChapterId();// + File.separator + pathName;
        File filePath = new File(tempPath);
        if(!filePath.exists()){
            filePath.mkdirs();
        }
        if(ppt){
            String pptPath =  global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                    + global.getCOURSE_LOCATION() + File.separator
                    + path + bFile.getChapterId() + File.separator + pathName;
            File pptDirectory = new File(pptPath);
            FileUtils.moveDirectory(pptDirectory,filePath);
        }else{
            String oldPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator + url;
            try {
                int bytesum = 0;
                int byteread = 0;
                File oldfile = new File(oldPath);
                if (oldfile.exists()) { //文件存在时
                    InputStream inStream = new FileInputStream(oldPath); //读入原文件
                    FileOutputStream fs = new FileOutputStream(tempPath + File.separator + pathName + File.separator+fileName);
                    byte[] buffer = new byte[1024000];
                    int length;
                    while ( (byteread = inStream.read(buffer)) != -1) {
                        bytesum += byteread; //字节数 文件大小
                        fs.write(buffer, 0, byteread);
                    }
                    inStream.close();
                    fs.close();
                    oldfile.delete();
                }
            }
            catch (Exception e) {
                System.out.println("出错");
                e.printStackTrace();

            }
        }

    }

    /**
     * 上传视频到video目录
     * @param file
     * @param bFile
     * @return
     * @throws IOException
     */
    private BFile uploadVideo(MultipartFile file, BFile bFile) throws IOException {

        BufferedOutputStream stream = null;

        String path;
        String pathName;
        if(bFile.getLessonId()!=null&&!bFile.getLessonId().equals("")){
            path = global.getLESSON_LOCATION() + File.separator + bFile.getLessonId() + File.separator;
        }else{
            path = global.getSL_LOCATION() + File.separator + bFile.getSlId() + File.separator;
        }
        if (bFile.getType().equals("0")){
            pathName = global.getVIDEO_LOCATION();//视频路径
        }else if(bFile.getType().equals("1")){
            pathName = global.getFILE_LOCATION();//文件路径
        }else if(bFile.getType().equals("2")){
            pathName = global.getPPT_LOCATION();//ppt路径
        }else{
            pathName = global.getPDF_LOCATION();//pdf路径
        }

        String realPath = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                + global.getCOURSE_LOCATION() + File.separator
                + path + bFile.getChapterId() + File.separator
                + pathName + File.separator;
        File outFile=new File(realPath);

        if(!outFile.exists()){//判断保存路径是否存在，不存在新建
            outFile.mkdirs();
        }

        //上传文件
        String uploadFile = file.getOriginalFilename();

        //文件的真实名称
        String name = uploadFile.substring(uploadFile.lastIndexOf(File.separator) + 1,uploadFile.lastIndexOf('.'));

        // 截取上传文件的后缀
        String uploadFileSuffix = uploadFile.substring(uploadFile.lastIndexOf('.') + 1, uploadFile.length());

        // 设置文件名为乱码
        String uploadFileName = IdGen.uuid();

        String filename = uploadFileName + "." + uploadFileSuffix;

        stream = new BufferedOutputStream(new FileOutputStream(new File(realPath + filename)));
        byte[] bytes = file.getBytes();
        stream.write(bytes,0,bytes.length);

        String newPath = global.getCOURSE_LOCATION() + File.separator
                + path + bFile.getChapterId() + File.separator
                + pathName + File.separator;

        if(uploadFileSuffix.equals("ppt")||uploadFileSuffix.equals("pptx")){//ppt转pdf
            String inputFile = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                    + newPath+ filename;
            String outputFile = global.getBASE_UPLOAD_SERVER_LOCATION() + File.separator
                    + newPath + uploadFileName + ".pdf";
            PptToPdf pptToPdf = new PptToPdf();
            pptToPdf.pptToPdf(inputFile,outputFile);
        }
        bFile.setVideoName(name+ "." +uploadFileSuffix);//文件名


        if (bFile.getType().equals("0")){
            bFile.setVideoUrl(newPath+ filename);//视频路径
        }else if(bFile.getType().equals("1")){
            bFile.setFileUrl(newPath+ filename);//文件路径
        }else if(bFile.getType().equals("2")){
            bFile.setPptUrl(newPath + uploadFileName + ".pdf");//ppt路径
        }else{
            bFile.setPdfUrl(newPath+ filename);//pdf路径
        }

        stream.close();
        return bFile;
    }

    @Override
    public BFile queryFileUrlByChapterId(String chapterId) {
        return bFileRepository.queryFileUrlByChapterId(chapterId);
    }
}
