package org.rcisoft.business.texaminfo.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.texaminfo.entity.TExamInfo;
import org.rcisoft.business.texaminfo.service.TExamInfoService;

import java.util.List;

/**
* Created by yangzhaojun on 2018-1-15 19:49:07.
*/
@RestController
@RequestMapping("/examInfo")
public class TExamInfoController extends PaginationController<TExamInfo> {

    @Autowired
    private TExamInfoService tExamInfoServiceImpl;


    @ApiOperation(value="添加", notes="添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping
    public Result add(@Valid TExamInfo tExamInfo, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR.getCode(),
                bindingResult.getFieldError().getDefaultMessage());
        }
        tExamInfo.setToken(getToken());
        PersistModel data = tExamInfoServiceImpl.save(tExamInfo);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tExamInfo);
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @DeleteMapping("/{id:\\d+}")
    public Result delete(@PathVariable String id) {
        TExamInfo tExamInfo = new TExamInfo();
        tExamInfo.setBusinessId(id);
        tExamInfo.setToken(getToken());
        PersistModel data = tExamInfoServiceImpl.remove(tExamInfo);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/{id:\\d+}")
    public Result update(@Valid TExamInfo tExamInfo, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR.getCode(),
                bindingResult.getFieldError().getDefaultMessage());
        }
        tExamInfo.setToken(getToken());
        PersistModel data = tExamInfoServiceImpl.merge(tExamInfo);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tExamInfo);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @GetMapping("/{id:\\d+}")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tExamInfoServiceImpl.findById(id));
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @GetMapping(value = "/queryTExamInfoByPagination")
    public GridModel listByPagination(TExamInfo tExamInfo) {
        tExamInfo.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        tExamInfoServiceImpl.findAllByPagination(getPaginationUtility(), tExamInfo);
        return getGridModelResponse();
    }
}
