package org.rcisoft.business.texaminfo.service.impl;

import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.texaminfo.dao.TExamInfoRepository;
import org.rcisoft.business.texaminfo.entity.TExamInfo;
import org.rcisoft.business.texaminfo.service.TExamInfoService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by yangzhaojun on 2018-1-15 19:49:07.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TExamInfoServiceImpl implements TExamInfoService {

    @Autowired
    private TExamInfoRepository tExamInfoRepository;


    /**
    * 保存 tExamInfo
    * @param tExamInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(TExamInfo tExamInfo){
        tExamInfo.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(tExamInfo);
        int line = tExamInfoRepository.insertSelective(tExamInfo);
        log.info(UserUtil.getUserInfoProp(tExamInfo.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            tExamInfo.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param tExamInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(TExamInfo tExamInfo){
        UserUtil.setCurrentMergeOperation(tExamInfo);
        int line = tExamInfoRepository.logicalDelete(tExamInfo);
        log.info(UserUtil.getUserInfoProp(tExamInfo.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        tExamInfo.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 tExamInfo
    * @param tExamInfo
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TExamInfo tExamInfo){
        UserUtil.setCurrentMergeOperation(tExamInfo);
        int line = tExamInfoRepository.updateByPrimaryKeySelective(tExamInfo);
        log.info(UserUtil.getUserInfoProp(tExamInfo.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            tExamInfo.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 tExamInfo
    * @param id
    * @return
    */
    public TExamInfo findById(String id){
        return tExamInfoRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 tExamInfo
    * @param tExamInfo
    * @return
    */
    public List<TExamInfo> findAllByPagination(PageUtil<TExamInfo> paginationUtility,
        TExamInfo tExamInfo){
        return tExamInfoRepository.queryTExamInfos(tExamInfo);
    }

}
