package org.rcisoft.business.tpaperchapter.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.tpaperchapter.entity.TPaperChapter;
import org.rcisoft.business.tpaperchapter.service.TPaperChapterService;

import java.util.List;

/**
* Created by yangzhaojun on 2018-1-15 19:49:07.
*/
@RestController
@RequestMapping("/t/paper/chapter")
public class TPaperChapterController extends PaginationController<TPaperChapter> {

    @Autowired
    private TPaperChapterService tPaperChapterServiceImpl;


    @ApiOperation(value="添加", notes="添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping
    public Result add(@Valid TPaperChapter tPaperChapter, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR.getCode(),
                bindingResult.getFieldError().getDefaultMessage());
        }
        tPaperChapter.setToken(getToken());
        PersistModel data = tPaperChapterServiceImpl.save(tPaperChapter);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tPaperChapter);
    }

    @ApiOperation(value="逻辑删除", notes="逻辑删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "id", required = false, dataType = "varchar")})
    @DeleteMapping("/{id:\\d+}")
    public Result delete(@PathVariable String id) {
        TPaperChapter tPaperChapter = new TPaperChapter();
        tPaperChapter.setBusinessId(id);
        tPaperChapter.setToken(getToken());
        PersistModel data = tPaperChapterServiceImpl.remove(tPaperChapter);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/{id:\\d+}")
    public Result update(@Valid TPaperChapter tPaperChapter, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR.getCode(),
                bindingResult.getFieldError().getDefaultMessage());
        }
        tPaperChapter.setToken(getToken());
        PersistModel data = tPaperChapterServiceImpl.merge(tPaperChapter);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tPaperChapter);
    }

    @ApiOperation(value="查看单 ", notes="查看单 ")
    @GetMapping("/{id:\\d+}")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tPaperChapterServiceImpl.findById(id));
    }

    @ApiOperation(value="查看  集合", notes="查看单  集合")
    @GetMapping(value = "/queryTPaperChapterByPagination")
    public GridModel listByPagination(TPaperChapter tPaperChapter) {
        tPaperChapter.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        tPaperChapterServiceImpl.findAllByPagination(getPaginationUtility(), tPaperChapter);
        return getGridModelResponse();
    }
}
