package org.rcisoft.business.tpaperchapter.service.impl;

import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.tpaperchapter.dao.TPaperChapterRepository;
import org.rcisoft.business.tpaperchapter.entity.TPaperChapter;
import org.rcisoft.business.tpaperchapter.service.TPaperChapterService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by yangzhaojun on 2018-1-15 19:49:07.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TPaperChapterServiceImpl implements TPaperChapterService {

    @Autowired
    private TPaperChapterRepository tPaperChapterRepository;


    /**
    * 保存 tPaperChapter
    * @param tPaperChapter
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(TPaperChapter tPaperChapter){
        tPaperChapter.setCommonBusinessId();
        //增加操作
        UserUtil.setCurrentPersistOperation(tPaperChapter);
        int line = tPaperChapterRepository.insertSelective(tPaperChapter);
        log.info(UserUtil.getUserInfoProp(tPaperChapter.getToken(),UserUtil.USER_USERNAME)+"新增了ID为"+
            tPaperChapter.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param tPaperChapter
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(TPaperChapter tPaperChapter){
        UserUtil.setCurrentMergeOperation(tPaperChapter);
        tPaperChapter.setDeleted();
        int line = tPaperChapterRepository.logicalDelete(tPaperChapter);
        log.info(UserUtil.getUserInfoProp(tPaperChapter.getToken(),UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
        tPaperChapter.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 修改 tPaperChapter
    * @param tPaperChapter
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TPaperChapter tPaperChapter){
        UserUtil.setCurrentMergeOperation(tPaperChapter);
        int line = tPaperChapterRepository.updateByPrimaryKeySelective(tPaperChapter);
        log.info(UserUtil.getUserInfoProp(tPaperChapter.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            tPaperChapter.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 tPaperChapter
    * @param id
    * @return
    */
    public TPaperChapter findById(String id){
        return tPaperChapterRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 tPaperChapter
    * @param tPaperChapter
    * @return
    */
    public List<TPaperChapter> findAllByPagination(PageUtil<TPaperChapter> paginationUtility,
        TPaperChapter tPaperChapter){
        tPaperChapter.setStart();
        tPaperChapter.setNotDeleted();
        return tPaperChapterRepository.queryTPaperChapters(tPaperChapter);
    }

}
