package org.rcisoft.business.tquestion.controller;


/*固定导入*/
import com.sun.org.apache.regexp.internal.RE;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.json.JSONArray;
import org.rcisoft.business.tquestion.entity.TQuestionOptions;
import org.rcisoft.business.tquestion.service.impl.TQuestionServiceImpl;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.common.model.GridModel;
import org.rcisoft.core.exception.ServiceException;
import javax.validation.Valid;


import org.rcisoft.business.tquestion.entity.TQuestion;
import org.rcisoft.business.tquestion.service.TQuestionService;

import java.util.List;
import java.util.Map;

/**
* Created by yangzhaojun on 2018-1-15 19:49:07.
*/
@RestController
@RequestMapping("/tQuestion")
public class TQuestionController extends PaginationController<TQuestion> {

    @Autowired
    private TQuestionService tQuestionServiceImpl;


    @ApiOperation(value="添加", notes="添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result add(TQuestion tQuestion, List<Map<String,Object>> questionOptions) {
        PersistModel data = tQuestionServiceImpl.save(tQuestion,questionOptions);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
            tQuestion);
    }

    @ApiOperation(value = "新增选项",notes = "新增加一个选项")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "qid", value = "题目id", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "alias", value = "列别名", required = true, dataType = "varchar"),
            @ApiImplicitParam(name = "desc", value = "列描述", required = true, dataType = "varchar")})
    @PostMapping("/addOption")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result addoption(@PathVariable TQuestionOptions tQuestionOptions){
        PersistModel data = tQuestionServiceImpl.saveoption(tQuestionOptions);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                tQuestionOptions);
    }

    @ApiOperation(value="删除选项", notes="根据ID删除一个选项")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")})
    @PostMapping("/removeOption")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result deleteOption(@PathVariable String id) {
        return Result.builder(new PersistModel(tQuestionServiceImpl.removeOption(id,getToken())),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                id);
    }


    @ApiOperation(value="删除题目", notes="根据ID删除一个题")
    @ApiImplicitParam(name = "id", value = "businessId", required = true, dataType = "varchar")
    @PostMapping("/remove")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result delete(String id) {
        return Result.builder(new PersistModel(tQuestionServiceImpl.remove(id,getToken())),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        id);
    }

    @ApiOperation(value="批量删除", notes="根据ID批量删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "id数组", value = "businessId数组", required = true, dataType = "varchar")})
    @PostMapping("/removeByIds")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result deletes( String idInfoList ) {
        PersistModel data = new PersistModel(tQuestionServiceImpl.removeByIds(idInfoList));
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                idInfoList);
    }

    @ApiOperation(value="修改", notes="修改")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/{id:\\d+}")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result update(@Valid TQuestion tQuestion, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            throw new ServiceException(ResultServiceEnums.PARAMETER_ERROR.getCode(),
                bindingResult.getFieldError().getDefaultMessage());
        }
        tQuestion.setToken(getToken());
        PersistModel data = tQuestionServiceImpl.merge(tQuestion);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tQuestion);
    }

    @ApiOperation(value="唯一查找", notes="根据ID查找一条记录")
    @GetMapping("/one")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result detail(@PathVariable String id) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        tQuestionServiceImpl.findById(id));
    }

    @ApiOperation(value="根据条件分页查询", notes="根据URL中的参数分页查询")
    @ApiImplicitParams({@ApiImplicitParam(name = "qtype" , value = "题目类型" , required = false , dataType = "varchar"),
        @ApiImplicitParam(name = "qtitle" , value = "关键字" , required = false , dataType = "varchar")})
    @GetMapping(value = "/queryTQuestionByPagination")
    @PreAuthorize("hasRole('ROLE_1002')")
    public GridModel listByPagination(TQuestion tQuestion) {
        tQuestion.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        tQuestionServiceImpl.findAllByPagination(getPaginationUtility(), tQuestion);
        return getGridModelResponse();
    }

    @ApiOperation(value="根据条件查询", notes="根据URL中的参数查询全部")
    @ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")
    @GetMapping(value = "/queryTQestions")
    @PreAuthorize("hasRole('ROLE_1002')")
    public Result queryTQestion(TQuestion tQuestion) {
        tQuestion.setCreateBy(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                tQuestionServiceImpl.queryTQuestions(tQuestion));
    }
}
