package org.rcisoft.business.tquestion.service.impl;

import org.apache.commons.collections.MapUtils;
import org.json.JSONArray;
import org.rcisoft.business.tquestion.dao.TQuestionOptionsRepository;
import org.rcisoft.business.tquestion.entity.TQuestionOptions;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.tquestion.dao.TQuestionRepository;
import org.rcisoft.business.tquestion.entity.TQuestion;
import org.rcisoft.business.tquestion.service.TQuestionService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by yangzhaojun on 2018-1-15 19:49:07.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TQuestionServiceImpl implements TQuestionService {

    @Autowired
    private TQuestionRepository tQuestionRepository;

    @Autowired
    private TQuestionOptionsRepository tQuestionOptionsRepository;

    /**
    * 保存 tQuestion
    * @param tQuestion
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(TQuestion tQuestion, List<Map<String,Object>> questionOptions){
        List<TQuestionOptions> optionList = new ArrayList();
        //增加主表题干信息
        UserUtil.setCurrentPersistOperation(tQuestion);
        int line = tQuestionRepository.insertSelective(tQuestion);
        //增加选项信息
        for(Map<String,Object> map : questionOptions){
            TQuestionOptions tQuestionOptions = new TQuestionOptions();
            tQuestionOptions.setCommonBusinessId();
            tQuestionOptions.setQid(tQuestion.getQid());
            tQuestionOptions.setAlias(String.valueOf(map.get("alias")));
            tQuestionOptions.setDescribe(String.valueOf(map.get("describe")));
            optionList.add(tQuestionOptions);
        }

        tQuestionRepository.insertOptionsList(optionList);
        return new PersistModel(line);
    }

    /**
     * 增加选项
     * @param tQuestionOptions
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel saveoption(TQuestionOptions tQuestionOptions) {
        tQuestionOptions.setCommonBusinessId();
        int line = tQuestionOptionsRepository.insertSelective(tQuestionOptions);
        return new PersistModel(line);
    }

    /**
     * 删除选项
     * @param id
     * @param token
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int removeOption(String id, String token){
        TQuestionOptions tQuestionOptions = new TQuestionOptions();
        tQuestionOptions.setBusinessId(id);
//        UserUtil.setCurrentMergeOperation(tQuestion);
        int line = tQuestionOptionsRepository.deleteByPrimaryKey(id);
        log.info(UserUtil.getUserInfoProp(token,UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
                tQuestionOptions.getBusinessId()+"的信息");
        return line;
    }

    /**
    * 逻辑删除
    * @param id
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int remove(String id, String token){
        TQuestion tQuestion = new TQuestion();
        tQuestion.setBusinessId(id);
        UserUtil.setCurrentMergeOperation(tQuestion);
        int line = tQuestionRepository.logicalDelete(tQuestion);
        log.info(UserUtil.getUserInfoProp(token,UserUtil.USER_USERNAME)+"逻辑删除了ID为"+
            tQuestion.getBusinessId()+"的信息");
        return line;
    }

    /**
    * 修改 tQuestion
    * @param tQuestion
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TQuestion tQuestion){
        UserUtil.setCurrentMergeOperation(tQuestion);
        int line = tQuestionRepository.updateByPrimaryKeySelective(tQuestion);
        log.info(UserUtil.getUserInfoProp(tQuestion.getToken(),UserUtil.USER_USERNAME)+"修改了ID为"+
            tQuestion.getBusinessId()+"的信息");
        return new PersistModel(line);
    }

    /**
    * 根据id查询 tQuestion
    * @param id
    * @return
    */
    public TQuestion findById(String id){
        return tQuestionRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 tQuestion
    * @param tQuestion
    * @return
    */
    public List<TQuestion> findAllByPagination(PageUtil<TQuestion> paginationUtility,
        TQuestion tQuestion){
        tQuestion.setStart();
        tQuestion.setNotDeleted();
        return tQuestionRepository.queryTQuestionsByPagination(tQuestion);
    }

    @Override
    public List<TQuestion> queryTQuestions(TQuestion tQuestion) {
        return tQuestionRepository.queryTQuestions(tQuestion);
    }

    @Override
    public int removeByIds(String idInfoList) {
        return tQuestionRepository.deleteByIds(idInfoList);
    }


}
