package org.rcisoft.core.util;

import freemarker.template.Configuration;
import freemarker.template.Template;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.command.LxcCommand;
import org.rcisoft.core.result.CommandResult;
import org.rcisoft.core.result.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.FileWriter;
import java.util.Map;

/**
 * Created by lcy on 18/1/9.
 */
@Component
@Slf4j
public class FreemarkerUtil {

    @Autowired
    Configuration freemarkerConfiguration;

    /**
     * 根据模板构建文件
     * @param model
     * @param destPath
     * @param param
     */
    public CommandResult generatorFileFromModel(String model, String destPath, Map param){
        CommandResult commandResult = null;
        try {
            if(StringUtils.isEmpty(model) && StringUtils.isEmpty(destPath)) {
                log.error("目录错误 ...." + destPath);
                commandResult = new CommandResult(ResultCode.FAIL,"目录不存在",null);
                return commandResult;
            }
            File dirFile = new File(destPath);
            /*目录不存在*/
            if(!dirFile.exists())
                dirFile.mkdirs();
            Template template = freemarkerConfiguration.getTemplate("lxc" + File.separator + model + ".ftl"); // freeMarker template
            template.process(param,new FileWriter(new File(destPath + LxcCommand.LXC_SETTING_NAME)));
            commandResult = new CommandResult(ResultCode.SUCCESS,null,null);
        }catch (Exception e){
            log.error(e.getMessage());
            commandResult = new CommandResult(ResultCode.FAIL,e.getMessage(),null);
        }
        return  commandResult;
    }
}
